@@ -81,6 +81,7 @@ impl InputSrc {
 /// field has no effect for PLL1.
 pub struct PllCfg {
     pub enabled: bool,
+    // pub fractional: bool,
     pub pllp_en: bool,
     pub pllq_en: bool,
     pub pllr_en: bool,
@@ -96,6 +97,7 @@ impl Default for PllCfg {
         // todo: Different defaults for different variants.
         Self {
             enabled: true,
+            // fractional: false,
             pllp_en: true,
             pllq_en: false,
             pllr_en: false,
@@ -363,6 +365,7 @@ impl Clocks {
                 // PWR D3 domain control register (PWR D3 domain control register (PWR_D3CR))
                 pwr.d3cr
                     .modify(|_, w| unsafe { w.vos().bits(VosRange::VOS1 as u8) });
+                while pwr.d3cr.read().vosrdy().bit_is_clear() {}
 
                 // 2. Enable the SYSCFG clock in the RCC by setting the SYSCFGEN bit in the
                 // RCC_APB4ENR register.
@@ -935,7 +938,9 @@ impl Default for Clocks {
 
 #[cfg(not(feature = "h7b3"))]
 impl Clocks {
-    /// Full speed of 480Mhz, with VC0 range 0.
+    /// Full speed of 480Mhz, with VC0 range 0. Note that special consideration needs to be taken
+    /// when using low power modes (ie anything with wfe or wfi) in this mode; may need to manually
+    /// disable and re-enable it.
     pub fn full_speed() -> Self {
         Self {
             pll1: PllCfg {
